package com.example.flutter_1flow_sdk;

import android.content.BroadcastReceiver;
import android.content.Context;
import android.content.Intent;
import android.content.IntentFilter;
import android.os.Handler;
import android.util.Log;

import androidx.annotation.NonNull;

import io.flutter.embedding.engine.plugins.FlutterPlugin;
import io.flutter.plugin.common.MethodCall;
import io.flutter.plugin.common.MethodChannel;
import io.flutter.plugin.common.MethodChannel.MethodCallHandler;
import io.flutter.plugin.common.MethodChannel.Result;

import com.oneflow.analytics.OneFlow;

import org.json.JSONArray;
import org.json.JSONException;
import org.json.JSONObject;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.logging.LogManager;

/**
 * Flutter1flowSdkPlugin
 */
public class Flutter1flowSdkPlugin implements FlutterPlugin, MethodCallHandler {
    /// The MethodChannel that will the communication between Flutter and native Android
    ///
    /// This local reference serves to register the plugin with the Flutter Engine and unregister it
    /// when the Flutter Engine is detached from the Activity
    private MethodChannel channel;
    private Context appContext;

    @Override
    public void onAttachedToEngine(@NonNull FlutterPluginBinding flutterPluginBinding) {
        channel = new MethodChannel(flutterPluginBinding.getBinaryMessenger(), "flutter_1flow_sdk");
        channel.setMethodCallHandler(this);
        appContext = flutterPluginBinding.getApplicationContext();
        IntentFilter inf = new IntentFilter();
        inf.addAction("survey_finished");
        appContext.registerReceiver(listFetched,inf);
    }

    BroadcastReceiver listFetched = new BroadcastReceiver() {
        @Override
        public void onReceive(Context context, Intent intent) {
            if(intent.getAction().equalsIgnoreCase("survey_finished")){
                String surveyDetailJSONString = intent.getStringExtra("surveyDetail");
                try {
                    JSONObject surveyDetailJSONObj = new JSONObject(surveyDetailJSONString);
                    final Map surveyDetailMap = jsonToMap(surveyDetailJSONObj);
                    new Handler().postDelayed(new Runnable() {
                        @Override
                        public void run() {
                            channel.invokeMethod("oneFlowSurveyFinishNotification",surveyDetailMap);
                        }
                    }, 100);
                } catch (JSONException e) {
                    e.printStackTrace();
                }

            }
        }
    };
    @Override
    public void onMethodCall(@NonNull MethodCall call, @NonNull Result result) {
        if (call.method.equals("configure")) {
            String projectKey = call.argument("projectKey");
            boolean enableSurveys = call.argument("enableSurveys");
            configure(projectKey, enableSurveys);
            result.success(null);
        } else if (call.method.equals("logUser")) {
            String uniqueId = call.argument("uniqueId");
            Map userDetails = call.argument("userDetails");
            logUser(uniqueId, userDetails);
            result.success(null);
        } else if (call.method.equals("recordEventName")) {
            String eventName = call.argument("eventName");
            Map parameters = call.argument("parameters");
            recordEventName(eventName, parameters);
            result.success(null);
        } else if (call.method.equals("enableSurveys")) {
            boolean enableSurveys = call.argument("enableSurveys");
            enableSurveys(enableSurveys);
            result.success(null);
        } else if (call.method.equals("useFont")) {
            String path = call.argument("path");
            useFont(path);
            result.success(null);
        } else {
            result.notImplemented();
        }
    }

    private void logUser(String uniqueId, Map userDetails) {
        HashMap hashMap = null;
        if(userDetails != null){
            hashMap = new HashMap(userDetails);
        }
        OneFlow.logUser(uniqueId, hashMap);
    }

    private void recordEventName(String eventName, Map parameters) {
        HashMap hashMap = null;
        if(parameters != null){
            hashMap = new HashMap(parameters);
        }
        OneFlow.recordEvents(eventName, hashMap);
    }

    private void configure(String projectKey, boolean enableSurveys){
        OneFlow.configure(appContext,projectKey);
        OneFlow.shouldShowSurvey(enableSurveys);
        OneFlow.shouldPrintLog(true);
    }

    private void enableSurveys(boolean enableSurveys){
        OneFlow.shouldShowSurvey(enableSurveys);
    }

    private void useFont(String path){
        OneFlow.useFont(path);
    }

    @Override
    public void onDetachedFromEngine(@NonNull FlutterPluginBinding binding) {
        channel.setMethodCallHandler(null);
    }

    private static List jsonToList(JSONArray jsonArray) throws JSONException {
        List list = new ArrayList();
        for (int i = 0; i < jsonArray.length(); i++) {
            Object value = jsonArray.get(i);
            if (value instanceof JSONObject) {
                list.add(jsonToMap((JSONObject) value));
            } else if (value instanceof JSONArray) {
                list.add(jsonToList((JSONArray) value));
            } else if (value == JSONObject.NULL) {
                list.add(null);
            }else{
                list.add(value);
            }
        }
        return list;
    }
    private static Map jsonToMap(JSONObject jsonObject) throws JSONException {
        Map map = new HashMap();
        Iterator iterator = jsonObject.keys();
        while (iterator.hasNext()) {
            String key = (String) iterator.next();
            Object value = jsonObject.get(key);
            if (value instanceof JSONObject) {
                map.put(key, jsonToMap((JSONObject) value));
            } else if (value instanceof JSONArray) {
                map.put(key, jsonToList((JSONArray) value));
            } else if (value == null || value == JSONObject.NULL) {
                map.put(key,null);
            }else{
                map.put(key,value);
            }
        }
        return map;
    }

    private JSONObject toJsonObject(Map map) throws JSONException {
        JSONObject object = new JSONObject();
        Iterator<String > iter = map.keySet().iterator();
        while (iter.hasNext()) {
            String key = iter.next();
            Object value = map.get(key);
            if (value instanceof Map) {
                object.put(key, toJsonObject((Map) value));
            } else if (value instanceof List) {
                object.put(key, toJsonArray((List) value));
            } else if (value == null || value == JSONObject.NULL) {
                object.put(key, null);
            } else {
                object.put(key, value);
            }
        }
        return object;
    }
    private JSONArray toJsonArray(List list) throws JSONException {
        JSONArray array = new JSONArray();
        for (int idx = 0; idx < list.size(); idx++) {
            Object value = list.get(idx);
            if (value instanceof Map) {
                array.put(toJsonObject((Map) value));
            } else if (value instanceof List) {
                array.put(toJsonArray((List) value));
            } else if (value == null || value == JSONObject.NULL) {
                array.put(null);
            } else {
                array.put(value);
            }
        }
        return array;
    }
}
