#import "Flutter1flowSdkPlugin.h"
@import _1Flow;

@implementation Flutter1flowSdkPlugin
+ (void)registerWithRegistrar:(NSObject<FlutterPluginRegistrar>*)registrar {
    FlutterMethodChannel* channel = [FlutterMethodChannel
                                     methodChannelWithName:@"flutter_1flow_sdk"
                                     binaryMessenger:[registrar messenger]];
    Flutter1flowSdkPlugin* instance = [[Flutter1flowSdkPlugin alloc] init];
    [registrar addMethodCallDelegate:instance channel:channel];
    [[NSNotificationCenter defaultCenter] addObserverForName:@"survey_finished" object:nil queue:nil usingBlock:^(NSNotification *note) {
        NSDictionary *pushNotification = note.userInfo;
        dispatch_async(dispatch_get_main_queue(), ^{
            [channel invokeMethod:@"oneFlowSurveyFinishNotification" arguments:pushNotification];
        });
    }];
}

- (void)handleMethodCall:(FlutterMethodCall*)call result:(FlutterResult)result {
    if ([@"configure" isEqualToString:call.method]) {
        NSString *projectKey = [call.arguments valueForKey:@"projectKey"];
        BOOL enableSurveys = [call.arguments valueForKey:@"enableSurveys"];
        [self configure:projectKey enableSurveys:enableSurveys];
    } else if ([@"logUser" isEqualToString:call.method]) {
        NSString *uniqueId = [call.arguments valueForKey:@"uniqueId"];
        id userDetails = [call.arguments valueForKey:@"userDetails"];
        [self logUser:uniqueId userDetails:userDetails];
    } else if ([@"recordEventName" isEqualToString:call.method]) {
        NSString *eventName = [call.arguments valueForKey:@"eventName"];
        id parameters = [call.arguments valueForKey:@"parameters"];
        [self recordEventName:eventName parameters:parameters];
    }  else if ([@"enableSurveys" isEqualToString:call.method]) {
        BOOL enableSurveys = [call.arguments valueForKey:@"enableSurveys"];
        [self enableSurveys:enableSurveys];
    }else if ([@"useFont" isEqualToString:call.method]) {
        NSString *fontFamily = [call.arguments valueForKey:@"fontFamily"];
        [self useFont:fontFamily];
    } else  {
        result(FlutterMethodNotImplemented);
    }
}

-(void)logUser:(NSString *)uniqueId userDetails:(NSDictionary *)userDetails{
    [OneFlow logUser:uniqueId userDetails:userDetails];
}

-(void)recordEventName:(NSString *)eventName parameters:(NSDictionary *)parameters{
    [OneFlow recordEventName:eventName parameters:parameters];
}

-(void)configure:(NSString *)projectKey enableSurveys:(BOOL)enableSurveys{
    [OneFlow configure:projectKey];
    [OneFlow setEnableSurveys:enableSurveys];
}

-(void)useFont:(NSString *)fontFamily{
    [OneFlow useFontWithFontFamily:fontFamily];
}
-(void)enableSurveys:(BOOL)enableSurveys{
    [OneFlow setEnableSurveys:enableSurveys];
}
@end
